import psycopg2
from psycopg2.extras import DictCursor
from flask import g
import os

DATABASE_URL = os.environ.get('DATABASE_URL', "postgresql://codyuser:59166498@localhost/scraperdb")

def get_db():
    db = getattr(g, '_database', None)
    if db is None:
        db = g._database = psycopg2.connect(DATABASE_URL)
    return db

def init_db():
    with psycopg2.connect(DATABASE_URL) as conn:
        with conn.cursor() as cur:
            cur.execute('''
                CREATE TABLE IF NOT EXISTS users (
                    id SERIAL PRIMARY KEY,
                    username TEXT UNIQUE NOT NULL,
                    password TEXT NOT NULL
                )
            ''')
            cur.execute('''
                CREATE TABLE IF NOT EXISTS scrapers (
                    id SERIAL PRIMARY KEY,
                    name TEXT NOT NULL,
                    url TEXT NOT NULL,
                    selector_type TEXT NOT NULL,
                    selector TEXT NOT NULL,
                    interval INTEGER NOT NULL,
                    last_run REAL DEFAULT 0,
                    use_proxy BOOLEAN NOT NULL DEFAULT FALSE,
                    proxy TEXT,
                    use_rotating_proxies BOOLEAN NOT NULL DEFAULT FALSE,
                    use_direct_connection BOOLEAN NOT NULL DEFAULT FALSE,
                    user_id INTEGER NOT NULL,
                    FOREIGN KEY (user_id) REFERENCES users (id)
                )
            ''')

            cur.execute('''
                CREATE TABLE IF NOT EXISTS scraper_results (
                    id SERIAL PRIMARY KEY,
                    scraper_id INTEGER,
                    result TEXT,
                    scraped_at REAL,
                    FOREIGN KEY (scraper_id) REFERENCES scrapers (id)
                )
            ''')

            cur.execute('''
                CREATE TABLE IF NOT EXISTS users (
                    id SERIAL PRIMARY KEY,
                    username TEXT UNIQUE NOT NULL,
                    password TEXT NOT NULL
                )
            ''')

            cur.execute('''
                CREATE TABLE IF NOT EXISTS proxies (
                    id SERIAL PRIMARY KEY,
                    proxy TEXT NOT NULL,
                    user_id INTEGER NOT NULL,
                    UNIQUE(proxy, user_id),
                    FOREIGN KEY (user_id) REFERENCES users (id)
                )
            ''')

            # Create scraper_groups table
            cur.execute('''
                CREATE TABLE IF NOT EXISTS scraper_groups (
                    id SERIAL PRIMARY KEY,
                    name TEXT NOT NULL,
                    user_id INTEGER NOT NULL,
                    FOREIGN KEY (user_id) REFERENCES users (id)
                )
            ''')

            # Check if group_id column exists in scrapers table
            cur.execute("""
                SELECT column_name 
                FROM information_schema.columns 
                WHERE table_name='scrapers' AND column_name='group_id'
            """)
            if cur.fetchone() is None:
                # Add group_id column if it doesn't exist
                cur.execute('''
                    ALTER TABLE scrapers 
                    ADD COLUMN group_id INTEGER,
                    ADD FOREIGN KEY (group_id) REFERENCES scraper_groups (id)
                ''')

    conn.commit()

def close_db(e=None):
    db = getattr(g, '_database', None)
    if db is not None:
        db.close()
