import requests
from database import get_db
from lxml import html
import time
import random
import psycopg2
from psycopg2.extras import DictCursor

USER_AGENTS = [
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:89.0) Gecko/20100101 Firefox/89.0',
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1.1 Safari/605.1.15',
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
    'Mozilla/5.0 (iPhone; CPU iPhone OS 14_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.0 Mobile/15E148 Safari/604.1'
]

def get_multiple_proxies(count=3):
    db = get_db()
    with db.cursor(cursor_factory=psycopg2.extras.DictCursor) as cur:
        cur.execute('SELECT proxy FROM proxies ORDER BY RANDOM() LIMIT %s', (count,))
        proxies = cur.fetchall()
    return [proxy['proxy'] for proxy in proxies]

def get_random_proxy():
    db = get_db()
    with db.cursor(cursor_factory=psycopg2.extras.DictCursor) as cur:
        cur.execute('SELECT proxy FROM proxies')
        proxies = cur.fetchall()
    if proxies:
        return random.choice(proxies)['proxy']
    return None

def get_random_user_agent():
    return random.choice(USER_AGENTS)

def run_scraper(url, selector_type, selector, use_proxy=False, proxy=None, use_rotating_proxies=False, use_direct_connection=False):
    headers = {
        'User-Agent': get_random_user_agent()
    }

    proxies_to_try = []
    if use_rotating_proxies:
        proxies_to_try = get_multiple_proxies(3)
        if not proxies_to_try:
            return ["No proxies available for rotating proxies"]
    elif use_proxy and proxy:
        proxies_to_try = [proxy]
    
    for attempt, proxy in enumerate(proxies_to_try + [None] if proxies_to_try else [None] * 3, 1):
        try:
            if proxy:
                host, port, user, password = proxy.split(':')
                proxies = {
                    'http': f'http://{user}:{password}@{host}:{port}',
                    'https': f'http://{user}:{password}@{host}:{port}'
                }
            else:
                proxies = None

            time.sleep(random.uniform(1, 3))
            response = requests.get(url, headers=headers, proxies=proxies, timeout=10)
            response.raise_for_status()
            
            tree = html.fromstring(response.content)
            selectors = selector.split('|')
            results = []
            
            for single_selector in selectors:
                single_selector = single_selector.strip()
                if selector_type == 'xpath':
                    elements = tree.xpath(single_selector)
                elif selector_type == 'css':
                    elements = tree.cssselect(single_selector)
                else:
                    return ["Invalid selector type"]
                
                results.extend([element.text_content().strip() for element in elements])
            
            if not results:
                return ["No elements found matching the selectors"]
            
            return results
        
        except requests.exceptions.ProxyError:
            if attempt < len(proxies_to_try) + (1 if use_direct_connection else 0):
                continue
            return ["Proxy error: Unable to connect to any proxy server after multiple attempts"]
        except requests.exceptions.RequestException as e:
            if attempt < len(proxies_to_try) + (1 if use_direct_connection else 0):
                continue
            return [f"An error occurred: {e}"]

    return ["Failed to scrape after all attempts"]
